module weiop_slo
  !! WEI slowness operator
  !! for FK-FX downward continuation
  !! Modified for two vel. fields

  use wei_util
  use wei_pro
  use wei_S
  use wei_attr
  use wei_dump
  !---------------
  implicit none

  real, dimension(:,:,:,:), pointer, private :: spxy
  real, dimension(:,:,:,:), pointer, private :: ssxy
  real, private :: eds !! epsilon(ds)
contains
  !----------------------------------------------------------------
  !! one reference slowness
  integer function weiop_slo1_init() result(st)
    integer :: er
    character(len=128), parameter :: unit='weiop_slo_init'
#ifdef DEBUG
    if(wpro%dbg>0) call in(unit)
#endif
    !! do nothing
#ifdef DEBUG
    if(wpro%dbg>0) call out(unit)
#endif
    st=weiOK
  end function weiop_slo1_init
  !----------------------------------------------------------------  
  !! one reference slowness
  integer function weiop_slo1(X,Y) result(st)
    type(weslow), intent(inout) :: X,Y
    integer :: izs
    character(len=128), parameter :: unit='weiop_slo1'
#ifdef DEBUG
    if(wpro%dbg>0) call in(unit)
#endif

    !! spread the slowness in its PBA
    call weiS_push(X,Y)

    !! s(z): FKop slowness
    do izs=1,wpro%nzs
       slo%nr_s(izs) = 1 !! one reference p-slowness
       slo%nr_r(izs) = 1 !! one reference s-slowness

       slo%rs_s(izs,1) = weiS_mean(X,izs)
       slo%rs_r(izs,1) = weiS_mean(Y,izs)
    end do

    slo%si_s=1.
    slo%si_r=1.

#ifdef DEBUG
    if(wpro%dbg>0) call out(unit)
#endif
    st=weiOK
  end function weiop_slo1
  !----------------------------------------------------------------
  !! N reference slownesses
  integer function weiop_sloN_init() result(st)
    integer :: er
    character(len=128), parameter :: unit='weiop_sloN_init'
#ifdef DEBUG
    if(wpro%dbg>0) call in(unit)
#endif

    call weimem_add(4,wpro%amx%n*wpro%amy%n*wpro%ahx%n*wpro%ahy%n)
    allocate(spxy(     wpro%amx%n,wpro%amy%n,wpro%ahx%n,wpro%ahy%n),stat=er)
    if(er/=0) call seperr('cant allocate spxy')

    call weimem_add(4,wpro%amx%n*wpro%amy%n*wpro%ahx%n*wpro%ahy%n)
    allocate(ssxy(     wpro%amx%n,wpro%amy%n,wpro%ahx%n,wpro%ahy%n),stat=er)
    if(er/=0) call seperr('cant allocate ssxy')

    eds = epsilon(1.)

#ifdef DEBUG
    if(wpro%dbg>0) call out(unit)
#endif
    st=weiOK
  end function weiop_sloN_init
  !----------------------------------------------------------------  
  !! N reference slownesses
  integer function weiop_sloN(X,Y) result(st)
    type(weslow), intent(inout) :: X,Y
    integer :: izs,ifk
    real    :: dsp, so, sz, dss
    real    :: sl,su
    real    :: spmin,spmax
    real    :: ssmin,ssmax
    character(len=128), parameter :: unit='weiop_sloN'
#ifdef DEBUG
    if(wpro%dbg>0) call in(unit)
#endif

    !! spread the slowness in its PBAs    
    call weiS_push(X,Y)

    !! min/max slowness in the interval
    izsloop: do izs=1,wpro%nzs
       slo%nr_s(izs) = wpro%nfk
       slo%nr_r(izs) = wpro%nfk

       spmin = weiS_min(X,izs)
       spmax = weiS_max(X,izs)
       dsp = (spmax-spmin) / wpro%nfk

       ssmin = weiS_min(Y,izs)
       ssmax = weiS_max(Y,izs)
       dss = (spmax-spmin) / wpro%nfk

       !! s(z): FKop slowness
       do ifk=1,wpro%nfk
          slo%rs_s(izs,ifk) = spmin + (ifk-.5)*dsp
          slo%rs_r(izs,ifk) = ssmin + (ifk-.5)*dss
       end do

       !! interpolation map
       slo%si_s(:,:,:,:,izs)=wpro%nfk
       slo%si_r(:,:,:,:,izs)=wpro%nfk

       !! source - P slowness
       spxy(:,:,:,:) = slo%ss_s(:,:,:,:,izs)

       do ifk=1,wpro%nfk                        !! [sl...sz...su]
          sz = slo%rs_s(izs,ifk)
          sl = sz-dsp/2-eds
          su = sz+dsp/2
          where((spxy >= sl) .and. (spxy < su))
             slo%si_s(:,:,:,:,izs)=ifk
          end where
       end do
       slo%sw_s(:,:,:,:,izs)=slo%si_s(:,:,:,:,izs)

       !! receiver - S slowness
       ssxy(:,:,:,:) = slo%ss_r(:,:,:,:,izs)

       do ifk=1,wpro%nfk                        !! [sl...sz...su]
          sz = slo%rs_r(izs,ifk)
          sl = sz-dss/2-eds
          su = sz+dss/2
          where((ssxy >= sl) .and. (ssxy < su))
             slo%si_r(:,:,:,:,izs)=ifk
          end where
       end do
       slo%sw_r(:,:,:,:,izs)=slo%si_r(:,:,:,:,izs)

    end do izsloop

#ifdef DEBUG
    if(wpro%dbg>0) call out(unit)
#endif
    st=weiOK
  end function weiop_sloN
  !----------------------------------------------------------------
end module weiop_slo



